#!/usr/bin/env/python
import sys
import os
import time
from . import get_key, authenticate, verify


def time_auth(d, msg_len=64):
    for k in d.keys():
        d[k] = [os.urandom(msg_len)]
    start = time.time()
    for (k, v) in d.items():
        v += [authenticate(k, v[0])]
    return float(len(d)) / (time.time() - start)


def time_verify(d):
    start = time.time()
    for (k, v) in d.items():
        verify(v[1], k, v[0])
    return float(len(d)) / (time.time() - start)


if __name__ == '__main__':
    try:
        NUM_ITER = int(sys.argv[1])
    except:
        NUM_ITER = 1000
    print('Iterations: ' + str(NUM_ITER))

    # get_key()
    global_d = {}
    start = time.time()
    for i in range(NUM_ITER):
        global_d[get_key()] = None
    key_persec = float(NUM_ITER) / (time.time() - start)

    # auth, verify short messages
    auth_persec_short = time_auth(global_d)
    verify_persec_short = time_verify(global_d)

    # reset keys
    global_d = {}
    for i in range(NUM_ITER):
        global_d[get_key()] = None
    # auth, verify long messages
    auth_persec_long = time_auth(global_d, msg_len=10000)
    verify_persec_long = time_verify(global_d)

    print(
        ('get_key /sec: %.0f'
         '\nauthenticate / sec (64-byte message): %.0f'
         '\nverify / sec (64-byte message): %.0f'
         '\nauthenticate / sec (10K message): %.0f'
         '\nverify / sec (10K message): %.0f'
         ) % (
             key_persec, auth_persec_short, verify_persec_short,
             auth_persec_long, verify_persec_long,))



from . import get_key, authenticate, verify, self_test


if __name__ == '__main__':
    print('Power-On Self Test result: ' + str(self_test()))
    msg = 'Hello world'
    print('\nTest class methods - authenticate, verify')
    kr = get_key()
    auth = authenticate(kr, msg)

    bad_kr = get_key()
    bad_msg = msg + '1'
    bad_auth = authenticate(kr, bad_msg)

    print('Good: %s\nBad auth: %s\nBad kr: %s\nBad msg: %s' % (
        str(verify(auth, kr, msg)),
        str(verify(bad_auth, kr, msg)),
        str(verify(auth, bad_kr, msg)),
        str(verify(auth, kr, bad_msg))
    ))



import sys
import os
from setuptools import setup, find_packages, Extension


os.chdir(os.path.dirname(sys.argv[0]) or ".")

'''
==============================================================================
PACKAGE DATA
==============================================================================
'''
# You _SHOULD_ set these
name = 'poly1305_donna'
version = '0.11.13'   # oldver: '0.11.12'
url = 'https://github.com/sundarnagarajan/py_poly1305-donna'
download_url = '%s/tree/%s' % (url, version)
description = name
install_requires = [
    'cffi_utils',
]
packages = find_packages()
license = 'License :: OSI Approved :: MIT License'

# The following are optional
author = 'Sundar Nagarajan'
# author_email = ''
maintainer = author
# maintainer_email = author_email
classifiers = [
    'Development Status :: 4 - Beta',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: Implementation :: PyPy',
    'License :: OSI Approved :: MIT License',
]
zip_safe = True


'''
==============================================================================
C EXTENSION DETAILS
==============================================================================
'''
c_dir = 'floodberry.poly1305_donna'
libname = 'libpoly1305donna'
c_src_files = [
    'poly1305-donna.c',
]
libpath = os.path.join(name, libname)
c_src_list = [os.path.join(c_dir, x) for x in c_src_files]
ext_modules = [
    Extension(
        name=libpath,
        sources=c_src_list,
        include_dirs=[c_dir],
    )
]


'''
==============================================================================
ADDITIONAL DATA FILES
==============================================================================
'''

data_dirs = [
    'doc',
    'floodberry.poly1305_donna',
]


'''
==============================================================================
ADDITIONAL keyword args to setup() - should not be required, normally
==============================================================================
'''
ADDL_KWARGS = dict(
)


'''
==============================================================================
           DO NOT CHANGE ANYTHING BELOW THIS
==============================================================================
'''


def get_longdesc(default=''):
    '''
    Returns-->str
    '''
    files = ['README.rst', 'README.md', 'README.txt', 'README']
    for f in files:
        try:
            return open(f, 'r').read()
        except:
            continue
    return default


def get_dirtree(topdir, dirlist=[]):
    '''
    topdir-->str: must be name of a dir under current working dir
    dirlist-->list of str: must all be names of dirs under topdir
    '''
    ret = []
    curdir = os.getcwd()
    if not os.path.isdir(topdir):
        return ret
    os.chdir(topdir)
    try:
        for dirname in dirlist:
            if not os.path.isdir(dirname):
                continue
            for (d, ds, fs) in os.walk(dirname):
                for f in fs:
                    ret += [os.path.join(d, f)]
        return ret
    except:
        return ret
    finally:
        os.chdir(curdir)

# Make some keywords MANDATORY
for k in [
    'name', 'version', 'description', 'license',
]:
    if k not in locals():
        raise Exception('Missing mandatory keyword: ' + k)

# keywords that are computed from variables
dirlist = locals().get('data_dirs', None)
if isinstance(dirlist, list):
    package_dir = {name: name}
    package_data = {name: get_dirtree(topdir=name, dirlist=dirlist)}

long_description = get_longdesc(description)

known_keywords = [
    'name', 'version', 'packages', 'description', 'license',
    'install_requires', 'requires', 'setup_requires',
    'ext_modules', 'package_dir', 'package_data',
    'zip_safe', 'classifiers', 'keywords',
    'long_description', 'url', 'download_url',
    'author', 'author_email', 'maintainer', 'maintainer_email',
]

kwdict = {}
for k in known_keywords:
    if k in locals():
        kwdict[k] = locals()[k]

# Additional keywords specified by user - shouldn't be required, normally
kwdict.update(ADDL_KWARGS)


setup(**kwdict)



import os
from cffi_utils.utils2to3 import toBytes
from cffi_utils.sowrapper import get_lib_ffi_resource


c_hdr = '''
void poly1305_auth(unsigned char mac[16],
                   const unsigned char *m,
                   size_t bytes,
                   const unsigned char key[32]);

int poly1305_verify(const unsigned char mac1[16],
                    const unsigned char mac2[16]);

int poly1305_power_on_self_test(void);
'''
module_name = 'poly1305_donna'
libpath = 'libpoly1305donna.so'


(ffi, lib) = get_lib_ffi_resource(
    module_name=module_name, libpath=libpath, c_hdr=c_hdr)


def self_test():
    '''
    Returns-->boolean
    '''
    return (lib.poly1305_power_on_self_test() == 1)


def get_key(random_fn=None):
    '''
    random_fn-->callable: takes one int param: number of random bytes
                Preferably leave unset; os.urandom() will be used
    Returns-->bytes: Secure key
    '''
    if random_fn is None:
        random_fn = os.urandom
    kr = ffi.new('unsigned char[32]', random_fn(32))
    return ffi.get_bytes(kr)


def authenticate(kr, msg, random_fn=None):
    '''
    kr-->bytes: Secure key - returned by get_key()
    msg-->str or bytes: Message to be authenticated
    random_fn-->callable: takes one int param: number of random bytes
                Preferably leave unset; os.urandom() will be used
    Returns-->bytes: auth
    '''
    if random_fn is None:
        random_fn = os.urandom
    auth = ffi.new('unsigned char[16]')
    msg = toBytes(msg)

    lib.poly1305_auth(auth, msg, len(msg), kr)
    return ffi.get_bytes(auth)


def verify(auth, kr, msg, random_fn=None):
    '''
    auth-->bytes
    kr-->bytes: Secure key - returned by get_key()
    msg-->str or bytes: Message to be authenticated
    Returns-->boolean
    '''
    (auth, kr, msg) = (toBytes(auth), toBytes(kr), toBytes(msg))
    auth2 = authenticate(kr, msg, random_fn=random_fn)
    return (lib.poly1305_verify(auth, auth2) == 1)
